#ifndef BACKHAUL_TOPOLOGY_H
#define BACKHAUL_TOPOLOGY_H

#include <libubox/list.h>
#include <stdint.h>

#define IFACE_MAX_NUM 16
#define NEIGHBORS_MAX_NUM 16


struct local_iface {
	uint8_t macaddr[6];
	uint16_t media_type;
	uint8_t number_of_neighbors;
	uint8_t neighbors_al_macs[NEIGHBORS_MAX_NUM][6];
};

struct bh_topology_dev;
#define UNKNOWN_TREE_LEVEL -1
struct backhaul_info {
	const struct bh_topology_dev *parent_in_tree;
	const struct local_iface *parent_iface;
	const struct local_iface *own_iface;
	int8_t level_in_tree;
};

/** Represents single device and its relation to neighbors in BH tree.
 *
 *  When bh_info is populated the list of bh_topology_dev can be traversed
 *  and for each device bh link defined as below can be read.
 *
 *  TR-181 BH link definition:
 *  BHDevALMac/BHLocalMac <-LinkType -< LocalMac (EM device)
 */
struct bh_topology_dev {
	struct list_head list_element;

	uint8_t al_macaddr[6];
	uint8_t number_of_interfaces;
	struct local_iface ifaces[IFACE_MAX_NUM];

	struct backhaul_info bh_info;
};

/** Used to store all discovered easy mesh devices and model tree topology. */
struct bh_topology_dev_list {
	struct list_head list_head; /* list of struct bh_topology_dev */
	uint32_t bh_topo_devs_number;
	bool bh_topology_updated;
};

struct bh_topology_dev *find_bh_topology_device(const uint8_t *al_macaddr,
						const struct bh_topology_dev_list *bh_topo_dev_list);

struct bh_topology_dev *add_bh_topology_device(const uint8_t *al_macaddr,
					       struct bh_topology_dev_list *bh_topo_dev_list);

void remove_bh_topology_device(const uint8_t *al_macaddr,
			       struct bh_topology_dev_list *bh_topo_dev_list);

void remove_all_bh_topology_devices(struct bh_topology_dev_list *bh_topo_dev_list);


/**
 * Builds backhaul topolgy tree from unordered list of bh_topology_dev structs
 * by setting backhaul_info fields.
 *
 * Controller is always root node in tree. Edges between tree nodes represent
 * BH links, parent nodes provides BH links to child node - each child node
 * can have only one parent (be connected over one BH link).
 */
void set_backhaul_tree_relations(const uint8_t *ctrl_al_macaddr,
				 struct bh_topology_dev_list *bh_topo_dev_list);

#endif /* BACKHAUL_TOPOLOGY_H */
